[systemd-commits] src/libsystemd-bus

Daniel Mack zonque at kemper.freedesktop.org
Tue Dec 17 04:46:29 PST 2013


 src/libsystemd-bus/bus-kernel.c |   24 ++++++++++++------
 src/libsystemd-bus/kdbus.h      |   51 ++++++++--------------------------------
 2 files changed, 27 insertions(+), 48 deletions(-)

New commits:
commit 816a3f9ec58a695a859aa9718c245a9476d72654
Author: Daniel Mack <zonque at gmail.com>
Date:   Tue Dec 17 13:45:02 2013 +0100

    bus: sync with kdbus
    
    kdbus_cmd_{ep,ns,bus}_make are now consolidated, and bloom_size has
    become an item of the dynamically sized list.

diff --git a/src/libsystemd-bus/bus-kernel.c b/src/libsystemd-bus/bus-kernel.c
index f32f271..5be91d4 100644
--- a/src/libsystemd-bus/bus-kernel.c
+++ b/src/libsystemd-bus/bus-kernel.c
@@ -1066,7 +1066,7 @@ int kdbus_translate_attach_flags(uint64_t mask, uint64_t *kdbus_mask) {
 }
 
 int bus_kernel_create_bus(const char *name, char **s) {
-        struct kdbus_cmd_bus_make *make;
+        struct kdbus_cmd_make *make;
         struct kdbus_item *n;
         int fd;
 
@@ -1077,19 +1077,27 @@ int bus_kernel_create_bus(const char *name, char **s) {
         if (fd < 0)
                 return -errno;
 
-        make = alloca0(ALIGN8(offsetof(struct kdbus_cmd_bus_make, items) +
+        make = alloca0(ALIGN8(offsetof(struct kdbus_cmd_make, items) +
+                              offsetof(struct kdbus_item, data64) + sizeof(uint64_t) +
                               offsetof(struct kdbus_item, str) +
                               DECIMAL_STR_MAX(uid_t) + 1 + strlen(name) + 1));
 
+        make->size = offsetof(struct kdbus_cmd_make, items);
+
         n = make->items;
+        n->size = offsetof(struct kdbus_item, data64) + sizeof(uint64_t);
+        n->type = KDBUS_ITEM_BLOOM_SIZE;
+        n->data64[0] = BLOOM_SIZE;
+        assert_cc(BLOOM_SIZE % 8 == 0);
+        make->size += ALIGN8(n->size);
+
+        n = KDBUS_ITEM_NEXT(n);
         sprintf(n->str, "%lu-%s", (unsigned long) getuid(), name);
         n->size = offsetof(struct kdbus_item, str) + strlen(n->str) + 1;
         n->type = KDBUS_ITEM_MAKE_NAME;
+        make->size += ALIGN8(n->size);
 
-        make->size = ALIGN8(offsetof(struct kdbus_cmd_bus_make, items) + n->size);
         make->flags = KDBUS_MAKE_POLICY_OPEN;
-        make->bloom_size = BLOOM_SIZE;
-        assert_cc(BLOOM_SIZE % 8 == 0);
 
         if (ioctl(fd, KDBUS_CMD_BUS_MAKE, make) < 0) {
                 close_nointr_nofail(fd);
@@ -1169,7 +1177,7 @@ int bus_kernel_create_starter(const char *bus, const char *name) {
 }
 
 int bus_kernel_create_namespace(const char *name, char **s) {
-        struct kdbus_cmd_ns_make *make;
+        struct kdbus_cmd_make *make;
         struct kdbus_item *n;
         int fd;
 
@@ -1180,7 +1188,7 @@ int bus_kernel_create_namespace(const char *name, char **s) {
         if (fd < 0)
                 return -errno;
 
-        make = alloca0(ALIGN8(offsetof(struct kdbus_cmd_ns_make, items) +
+        make = alloca0(ALIGN8(offsetof(struct kdbus_cmd_make, items) +
                               offsetof(struct kdbus_item, str) +
                               strlen(name) + 1));
 
@@ -1189,7 +1197,7 @@ int bus_kernel_create_namespace(const char *name, char **s) {
         n->size = offsetof(struct kdbus_item, str) + strlen(n->str) + 1;
         n->type = KDBUS_ITEM_MAKE_NAME;
 
-        make->size = ALIGN8(offsetof(struct kdbus_cmd_ns_make, items) + n->size);
+        make->size = ALIGN8(offsetof(struct kdbus_cmd_make, items) + n->size);
         make->flags = KDBUS_MAKE_POLICY_OPEN | KDBUS_MAKE_ACCESS_WORLD;
 
         if (ioctl(fd, KDBUS_CMD_NS_MAKE, make) < 0) {
diff --git a/src/libsystemd-bus/kdbus.h b/src/libsystemd-bus/kdbus.h
index 972a02d..e9ffecd 100644
--- a/src/libsystemd-bus/kdbus.h
+++ b/src/libsystemd-bus/kdbus.h
@@ -201,6 +201,7 @@ struct kdbus_policy {
  * @KDBUS_ITEM_PAYLOAD_MEMFD:	Data as sealed memfd
  * @KDBUS_ITEM_FDS:		Attached file descriptors
  * @KDBUS_ITEM_BLOOM:		For broadcasts, carries bloom filter
+ * @KDBUS_ITEM_BLOOM_SIZE:	Desired bloom size, used by KDBUS_CMD_BUS_MAKE
  * @KDBUS_ITEM_DST_NAME:	Destination's well-known name
  * @KDBUS_ITEM_PRIORITY:	Queue priority for message
  * @KDBUS_ITEM_MAKE_NAME:	Name of namespace, bus, endpoint
@@ -233,6 +234,7 @@ enum kdbus_item_type {
 	KDBUS_ITEM_PAYLOAD_MEMFD,
 	KDBUS_ITEM_FDS,
 	KDBUS_ITEM_BLOOM,
+	KDBUS_ITEM_BLOOM_SIZE,
 	KDBUS_ITEM_DST_NAME,
 	KDBUS_ITEM_PRIORITY,
 	KDBUS_ITEM_MAKE_NAME,
@@ -493,46 +495,15 @@ enum kdbus_make_flags {
 };
 
 /**
  * struct kdbus_cmd_bus_make - struct to make a bus
  * @size:		The total size of the struct
- * @flags:		Properties for the bus to create
- * @bloom_size:		Size of the bloom filter for this bus
- * @items:		Items describing details such as the name of the bus
+ * @flags:		Properties for the bus/ep/ns to create
+ * @items:		Items describing details
  *
- * This structure is used with the KDBUS_CMD_BUS_MAKE ioctl.
+ * This structure is used with the KDBUS_CMD_BUS_MAKE, KDBUS_CMD_EP_MAKE and
+ * KDBUS_CMD_NS_MAKE ioctls.
  */
-struct kdbus_cmd_bus_make {
-	__u64 size;
-	__u64 flags;
-	__u64 bloom_size;
-	struct kdbus_item items[0];
-} __attribute__((aligned(8)));
-
-/**
- * struct kdbus_cmd_ep_make - struct to make an endpoint
- * @size:		The total size of the struct
- * @flags:		Unused for now
- * @items:		Items describing details such as the
- * 			name of the endpoint
- *
- * This structure is used with the KDBUS_CMD_EP_MAKE ioctl.
- */
-struct kdbus_cmd_ep_make {
-	__u64 size;
-	__u64 flags;
-	struct kdbus_item items[0];
-} __attribute__((aligned(8)));
-
-/**
- * struct kdbus_cmd_ns_make - struct to make a namespace
- * @size:		The total size of the struct
- * @flags:		Unused for now
- * @items:		Items describing details such as the
- * 			name of the namespace
- *
- * This structure is used with the KDBUS_CMD_NS_MAKE ioctl.
- */
-struct kdbus_cmd_ns_make {
+struct kdbus_cmd_make {
 	__u64 size;
 	__u64 flags;
 	struct kdbus_item items[0];
@@ -772,9 +743,9 @@ struct kdbus_cmd_match {
  * 				be changed as long as the file is shared.
  */
 enum kdbus_ioctl_type {
-	KDBUS_CMD_BUS_MAKE =		_IOW (KDBUS_IOC_MAGIC, 0x00, struct kdbus_cmd_bus_make),
-	KDBUS_CMD_NS_MAKE =		_IOR (KDBUS_IOC_MAGIC, 0x10, struct kdbus_cmd_ns_make),
-	KDBUS_CMD_EP_MAKE =		_IOW (KDBUS_IOC_MAGIC, 0x20, struct kdbus_cmd_ep_make),
+	KDBUS_CMD_BUS_MAKE =		_IOW (KDBUS_IOC_MAGIC, 0x00, struct kdbus_cmd_make),
+	KDBUS_CMD_NS_MAKE =		_IOR (KDBUS_IOC_MAGIC, 0x10, struct kdbus_cmd_make),
+	KDBUS_CMD_EP_MAKE =		_IOW (KDBUS_IOC_MAGIC, 0x20, struct kdbus_cmd_make),
 
 	KDBUS_CMD_HELLO =		_IOWR(KDBUS_IOC_MAGIC, 0x30, struct kdbus_cmd_hello),
 
@@ -493,46 +495,15 @@ enum kdbus_make_flags {
 
 /**
- * struct kdbus_cmd_bus_make - struct to make a bus
+ * struct kdbus_cmd_make - struct to make a bus, an endpoint or a namespace
  * @size:		The total size of the struct



More information about the systemd-commits mailing list